Attribute VB_Name = "Win32DebugAPI"
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' John Robbins
' Microsoft Systems Journal - August, 1997
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FILE         :     WIN32DebugAPI.bas
' DESCRIPTION  :
'  This module defines the structures, types, and APIs that make up the
'  WIN32 Debug API.  Since VB does not yet support unions, some of the
'  structure definitions here might look a little strange.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Option Explicit

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                         Constants and defines
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Const DEBUG_ONLY_THIS_PROCESS = &H2

Public Const DBG_CONTINUE = &H10002
Public Const DBG_EXCEPTION_NOT_HANDLED = &H80010001

Public Const EXCEPTION_DEBUG_EVENT = 1
Public Const CREATE_THREAD_DEBUG_EVENT = 2
Public Const CREATE_PROCESS_DEBUG_EVENT = 3
Public Const EXIT_THREAD_DEBUG_EVENT = 4
Public Const EXIT_PROCESS_DEBUG_EVENT = 5
Public Const LOAD_DLL_DEBUG_EVENT = 6
Public Const UNLOAD_DLL_DEBUG_EVENT = 7
Public Const OUTPUT_DEBUG_STRING_EVENT = 8
Public Const RIP_EVENT = 9

Public Const EXCEPTION_ACCESS_VIOLATION = &HC0000005
Public Const EXCEPTION_DATATYPE_MISALIGNMENT = &H80000002
Public Const EXCEPTION_BREAKPOINT = &H80000003
Public Const EXCEPTION_SINGLE_STEP = &H80000004
Public Const EXCEPTION_ARRAY_BOUNDS_EXCEEDED = &HC000008C
Public Const EXCEPTION_FLT_DENORMAL_OPERAND = &HC000008D
Public Const EXCEPTION_FLT_DIVIDE_BY_ZERO = &HC000008E
Public Const EXCEPTION_FLT_INEXACT_RESULT = &HC000008F
Public Const EXCEPTION_FLT_INVALID_OPERATION = &HC0000090
Public Const EXCEPTION_FLT_OVERFLOW = &HC0000091
Public Const EXCEPTION_INT_DIVIDE_BY_ZERO = &HC0000094
Public Const EXCEPTION_INT_OVERFLOW = &HC0000095
Public Const EXCEPTION_PRIV_INSTRUCTION = &HC0000096
Public Const EXCEPTION_IN_PAGE_ERROR = &HC0000006
Public Const EXCEPTION_ILLEGAL_INSTRUCTION = &HC000001D
Public Const EXCEPTION_NONCONTINUABLE_EXCEPTION = &HC0000025
Public Const EXCEPTION_STACK_OVERFLOW = &HC00000FD
Public Const EXCEPTION_INVALID_DISPOSITION = &HC0000026
Public Const EXCEPTION_GUARD_PAGE = &H80000001

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                             General Types
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Const EXCEPTION_MAXIMUM_PARAMETERS = 15

Type EXCEPTION_RECORD
   ExceptionCode As Long
   ExceptionFlags As Long
   ExceptionRecord As Long
   ExceptionAddress As Long
   NumberParameters As Long
   ExceptionInformation(EXCEPTION_MAXIMUM_PARAMETERS - 1) As Long
End Type

Type EXCEPTION_DEBUG_INFO
   ExceptionRecord As EXCEPTION_RECORD
   dwFirstChance As Long
End Type

Type CREATE_THREAD_DEBUG_INFO
   hThread As Long
   lpThreadLocalBase As Long
   lpStartAddress As Long
End Type

Type CREATE_PROCESS_DEBUG_INFO
   hFile As Long
   hProcess As Long
   hThread As Long
   lpBaseOfImage As Long
   dwDebugInfoFileOffset As Long
   nDebugInfoSize As Long
   lpThreadLocalBase As Long
   lpStartAddress As Long
   lpImageName As Long
   fUnicode As Integer
End Type

Type EXIT_THREAD_DEBUG_INFO
   dwExitCode As Long
End Type

Type EXIT_PROCESS_DEBUG_INFO
   dwExitCode As Long
End Type

Type LOAD_DLL_DEBUG_INFO
   hFile As Long
   lpBaseOfDll As Long
   dwDebugInfoFileOffset As Long
   nDebugInfoSize As Long
   lpImageName As Long
   fUnicode As Integer
End Type

Type UNLOAD_DLL_DEBUG_INFO
   lpBaseOfDll As Long
End Type

Type OUTPUT_DEBUG_STRING_INFO
   lpDebugStringData As Long
   fUnicode As Integer
   nDebugStringLength As Integer
End Type

Type RIP_INFO
   dwError As Long
   dwType As Long
End Type

' The DEBUG_EVENT structure is one giant C union.  Since VB cannot deal
'  with these, you have to be a little creative to get at the data.
'  I just declared the union data portion of the DEBUG_EVENT as an
'  EXCEPTION_DEBUG_INFO, which is the larges of the unioned
'  structures.  To get at the individual unioned structures, use the
'  LSet statement to essentially do a memcpy from the dwUnionData into
'  the appropriate structure based on dwDebugEventCode.  For example:
'
' Dim stEvtDbg As DEBUG_EVENT
' Dim stCreateProcess AS CREATE_PROCESS_DEBUG_INFO
' WaitForDebugEvent ( stEvtDbg , INFINITE )
' If ( CREATE_PROCESS_DEBUG_EVENT = stEvtDbg.dwDebugEventCode ) then
'    ' Here's the magic LSet.
'    LSet stCreateProcess = stEvtDbg.dwUnionData
' End If
'
' It is not a good idea to try and pick the information directly out
'  of the dwUnionData field.

Type DEBUG_EVENT
   dwDebugEventCode As Long
   dwProcessID As Long
   dwThreadID As Long
   ' Pad out all the union data.  The EXCEPTION_DEBUG_INFO is the
   '  largest of the unioned data structures.
   dwUnionData As EXCEPTION_DEBUG_INFO
End Type

' Since VB does not have a straight 64 bit datatype other than double,
'  I use the large integer type for all the 64 bit ALPHA registers.  You
'  don't really want to be trying bit manipulation on floating point
'  types.
Type LARGE_INTEGER
   lowpart As Long
   highpart As Long
End Type

' Those pesky context structures.
#If ALPHA Then

Public Const CONTEXT_ALPHA = &H20000

Public Const CONTEXT_CONTROL = &H20001
Public Const CONTEXT_FLOATING_POINT = &H20002
Public Const CONTEXT_INTEGER = &H20004

Type CONTEXT
   FltF0 As LARGE_INTEGER
   FltF1 As LARGE_INTEGER
   FltF2 As LARGE_INTEGER
   FltF3 As LARGE_INTEGER
   FltF4 As LARGE_INTEGER
   FltF5 As LARGE_INTEGER
   FltF6 As LARGE_INTEGER
   FltF7 As LARGE_INTEGER
   FltF8 As LARGE_INTEGER
   FltF9 As LARGE_INTEGER
   FltF10 As LARGE_INTEGER
   FltF11 As LARGE_INTEGER
   FltF12 As LARGE_INTEGER
   FltF13 As LARGE_INTEGER
   FltF14 As LARGE_INTEGER
   FltF15 As LARGE_INTEGER
   FltF16 As LARGE_INTEGER
   FltF17 As LARGE_INTEGER
   FltF18 As LARGE_INTEGER
   FltF19 As LARGE_INTEGER
   FltF20 As LARGE_INTEGER
   FltF21 As LARGE_INTEGER
   FltF22 As LARGE_INTEGER
   FltF23 As LARGE_INTEGER
   FltF24 As LARGE_INTEGER
   FltF25 As LARGE_INTEGER
   FltF26 As LARGE_INTEGER
   FltF27 As LARGE_INTEGER
   FltF28 As LARGE_INTEGER
   FltF29 As LARGE_INTEGER
   FltF30 As LARGE_INTEGER
   FltF31 As LARGE_INTEGER
   IntV0 As LARGE_INTEGER
   IntT0 As LARGE_INTEGER
   IntT1 As LARGE_INTEGER
   IntT2 As LARGE_INTEGER
   IntT3 As LARGE_INTEGER
   IntT4 As LARGE_INTEGER
   IntT5 As LARGE_INTEGER
   IntT6 As LARGE_INTEGER
   IntT7 As LARGE_INTEGER
   IntS0 As LARGE_INTEGER
   IntS1 As LARGE_INTEGER
   IntS2 As LARGE_INTEGER
   IntS3 As LARGE_INTEGER
   IntS4 As LARGE_INTEGER
   IntS5 As LARGE_INTEGER
   IntFp As LARGE_INTEGER
   IntA0 As LARGE_INTEGER
   IntA1 As LARGE_INTEGER
   IntA2 As LARGE_INTEGER
   IntA3 As LARGE_INTEGER
   IntA4 As LARGE_INTEGER
   IntA5 As LARGE_INTEGER
   IntT8 As LARGE_INTEGER
   IntT9 As LARGE_INTEGER
   IntT10 As LARGE_INTEGER
   IntT11 As LARGE_INTEGER
   IntRa As LARGE_INTEGER
   IntT12 As LARGE_INTEGER
   IntAt As LARGE_INTEGER
   IntGp As LARGE_INTEGER
   IntSp As LARGE_INTEGER
   IntZero As LARGE_INTEGER
   Fpcr As LARGE_INTEGER
   SoftFpcr As LARGE_INTEGER
   Fir As LARGE_INTEGER
   Psr As Long
   ContextFlags As Long
   Fill(4) As Long
End Type


#Else

' INTEL Context information.
Public Const SIZE_OF_80387_REGISTERS = 80

Public Const CONTEXT_i386 = &H10000
Public Const CONTEXT_i486 = &H10000

Public Const CONTEXT_CONTROL = &H10001
Public Const CONTEXT_INTEGER = &H10002
Public Const CONTEXT_SEGMENTS = &H10004
Public Const CONTEXT_FLOATING_POINT = &H10008
Public Const CONTEXT_DEBUG_REGISTERS = &H10010

Public Const CONTEXT_FULL = &H10007

Type FLOATING_SAVE_AREA
   ControlWord As Long
   StatusWord As Long
   TagWord As Long
   ErrorOffset As Long
   ErrorSelector As Long
   DataOffset As Long
   DataSelector As Long
   RegisterArea(SIZE_OF_80387_REGISTERS) As Byte
   Cr0NpxState As Long
End Type

Type CONTEXT
   ContextFlags As Long
   Dr0 As Long
   Dr1 As Long
   Dr2 As Long
   Dr3 As Long
   Dr6 As Long
   Dr7 As Long
   FloatSave As FLOATING_SAVE_AREA
   SegGs As Long
   SegFs As Long
   SegEs As Long
   SegDs As Long
   Edi As Long
   Esi As Long
   Ebx As Long
   Edx As Long
   Ecx As Long
   Eax As Long
   Ebp As Long
   Eip As Long
   SegCs As Long
   EFlags As Long
   Esp As Long
   SegSs As Long
End Type

#End If

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Debugging Functions
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Declare Function TerminateThread Lib "kernel32" _
      (ByVal hThread As Long, _
       ByVal dwExitCode As Long) As Long

Public Declare Function WaitForDebugEvent Lib "kernel32" _
      (ByRef lpDebugEvent As DEBUG_EVENT, _
       ByVal dwMilliseconds As Long) As Long

Public Declare Function ContinueDebugEvent Lib "kernel32" _
      (ByVal dwProcessID As Long, _
       ByVal dwThreadID As Long, _
       ByVal dwContinueStatus As Long) As Long

Public Declare Function GetThreadContext Lib "kernel32" _
      (ByVal hThread As Long, _
       ByRef lpContext As CONTEXT) As Long

Public Declare Function SetThreadContext Lib "kernel32" _
      (ByVal hThread As Long, _
       ByRef lpContext As CONTEXT) As Long



